import "core-js/modules/es6.promise";
import "core-js/modules/es7.symbol.async-iterator";
import "core-js/modules/es6.symbol";
import "core-js/modules/web.dom.iterable";
import "core-js/modules/es6.array.iterator";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import "core-js/modules/es6.regexp.search";
import "core-js/modules/es6.string.starts-with";
import "regenerator-runtime/runtime";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";

/**
 * @license
 * Copyright © 2017-2019 Moov Corporation.  All rights reserved.
 */
var installed = false;
var messageQueue = [];

var messageSw = function messageSw(payload) {
  if (installed) {
    if (navigator.serviceWorker.controller) {
      navigator.serviceWorker.controller.postMessage(payload);
    }
  } else {
    messageQueue.push(payload);
  }
};
/**
 * @license
 * Copyright © 2017-2019 Moov Corporation.  All rights reserved.
 */

/**
 * Cache content using the service worker.  If content is not supplied, the service worker will fetch
 * the content from the server
 * @param {String} path The URI path of the request
 * @param {String} cacheData The data to cache
 */


export function cache(path, cacheData) {
  var _ref = window.moov || {},
      apiVersion = _ref.apiVersion;

  if (window.moov.router && !window.moov.router.willCacheOnClient({
    path: path
  })) {
    // Never cache a path unless it matches a route with a cache handler
    // otherwise we could wind up caching pages like the cart that are not cacheable
    return;
  }

  if (cacheData) {
    messageSw({
      action: 'cache-state',
      path: path,
      apiVersion: apiVersion,
      cacheData: cacheData
    });
  } else {
    messageSw({
      action: 'cache-path',
      path: path,
      apiVersion: apiVersion
    });
  }
}
/**
 * Prefetches and caches JSON for the specified path
 * @param {String} path A URL path for a page (without .json)
 */

export function prefetchJsonFor(_x, _x2) {
  return _prefetchJsonFor.apply(this, arguments);
}
/**
 * Prefetches and caches SSR and JSON for the specified path
 * @param {String} path A URL path for a page (without .json)
 */

function _prefetchJsonFor() {
  _prefetchJsonFor = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee(path, includeSSR) {
    var url, _url;

    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (path) {
              _context.next = 2;
              break;
            }

            return _context.abrupt("return");

          case 2:
            if (path.startsWith('http')) {
              url = new URL(path);
              cache("".concat(url.origin).concat(url.pathname, ".json").concat(url.search));
            } else {
              _url = new URL("http://z.z".concat(path));
              cache("".concat(_url.pathname, ".json").concat(_url.search));
            }

            if (includeSSR) {
              cache(path);
            }

          case 4:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
  return _prefetchJsonFor.apply(this, arguments);
}

export function prefetch(path) {
  cache(path);
  prefetchJsonFor(path);
}
/**
 * Aborts all in progress prefetches.  Call this function to prevent prefetching from blocking
 * more important requests, like page navigation.
 */

export function abortPrefetches() {
  messageSw({
    action: 'abort-prefetches'
  });
}
/**
 * Resume queued prefetch requests which were cancelled to allow for more important requests
 */

export function resumePrefetches() {
  messageSw({
    action: 'resume-prefetches'
  });
}
/**
 * Configures runtime caching options
 * @private
 * @param {Object} options
 * @param {Object} options.cacheName The name of the runtime cache
 * @param {Object} options.maxEntries The max number of entries to store in the cache
 * @param {Object} options.maxAgeSeconds The TTL in seconds for entries
 */

export function configureCache(options) {
  messageSw({
    action: 'configure-runtime-caching',
    options: options
  });
}
/**
 * Clears all API and SSR responses from the client cache
 * @return {Promise} Resolved once all caches have b
 */

export function clearCache() {
  return _clearCache.apply(this, arguments);
}
/**
 * Resolves when the service worker has been installed
 * @private
 */

function _clearCache() {
  _clearCache = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee2() {
    var keys, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, key;

    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (!('caches' in window)) {
              _context2.next = 31;
              break;
            }

            _context2.next = 3;
            return caches.keys();

          case 3:
            keys = _context2.sent;
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context2.prev = 7;
            _iterator = keys[Symbol.iterator]();

          case 9:
            if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
              _context2.next = 17;
              break;
            }

            key = _step.value;

            if (key.startsWith('workbox-precache')) {
              _context2.next = 14;
              break;
            }

            _context2.next = 14;
            return caches.delete(key);

          case 14:
            _iteratorNormalCompletion = true;
            _context2.next = 9;
            break;

          case 17:
            _context2.next = 23;
            break;

          case 19:
            _context2.prev = 19;
            _context2.t0 = _context2["catch"](7);
            _didIteratorError = true;
            _iteratorError = _context2.t0;

          case 23:
            _context2.prev = 23;
            _context2.prev = 24;

            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }

          case 26:
            _context2.prev = 26;

            if (!_didIteratorError) {
              _context2.next = 29;
              break;
            }

            throw _iteratorError;

          case 29:
            return _context2.finish(26);

          case 30:
            return _context2.finish(23);

          case 31:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this, [[7, 19, 23, 31], [24,, 26, 30]]);
  }));
  return _clearCache.apply(this, arguments);
}

export function waitForServiceWorkerController() {
  return _waitForServiceWorkerController.apply(this, arguments);
}
/**
 * Removes runtime caches for old versions of the api.  This ensures that all responses
 * are appropriate for the current version of the UI.
 * @private
 */

function _waitForServiceWorkerController() {
  _waitForServiceWorkerController = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee3() {
    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            if (!(!navigator.serviceWorker || !navigator.serviceWorker.ready)) {
              _context3.next = 2;
              break;
            }

            return _context3.abrupt("return", false);

          case 2:
            return _context3.abrupt("return", new Promise(function (resolve) {
              navigator.serviceWorker.ready.then(function () {
                if (navigator.serviceWorker.controller) {
                  return resolve(true);
                }

                navigator.serviceWorker.addEventListener('controllerchange', function () {
                  return resolve(true);
                });
              });
            }));

          case 3:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));
  return _waitForServiceWorkerController.apply(this, arguments);
}

export function removeOldCaches() {
  if (window.moov && window.moov.apiVersion) {
    messageSw({
      action: 'remove-old-caches',
      apiVersion: window.moov.apiVersion
    });
  }
}
export function isServiceWorkerReady() {
  return installed;
}
export default {
  removeOldCaches: removeOldCaches,
  waitForServiceWorkerController: waitForServiceWorkerController,
  configureCache: configureCache,
  resumePrefetches: resumePrefetches,
  abortPrefetches: abortPrefetches,
  prefetch: prefetch,
  prefetchJsonFor: prefetchJsonFor,
  isServiceWorkerReady: isServiceWorkerReady,
  cache: cache
};

if (typeof window !== 'undefined') {
  waitForServiceWorkerController().then(function () {
    installed = true;
    messageQueue.map(messageSw);
    messageQueue = [];
  });
}
//# sourceMappingURL=serviceWorker.js.map